(require 'ert)

;; Timeout in seconds; the test fails if the timeout is reached.
(defvar sentinel-test-wait-timeout 2.0)

;; Start a process that exits immediately.  Call WAIT-FUNCTION,
;; possibly multiple times, to wait for the process to complete.
(defun sentinel-test-wait-function-working-p (wait-function)
  (let ((proc (start-process "test" nil "bash" "-c" "exit 20"))
	(sentinel-called nil)
	(start-time (float-time)))
    (set-process-sentinel proc (lambda (proc msg)
				 (setq sentinel-called t)))
    (while (not (or sentinel-called
		    (> (- (float-time) start-time)
		       sentinel-test-wait-timeout)))
      (funcall wait-function))
    (cl-assert (eq (process-status proc) 'exit))
    (cl-assert (= (process-exit-status proc) 20))
    sentinel-called))

(ert-deftest sentinel-test-accept-process-output ()
  (should (sentinel-test-wait-function-working-p #'accept-process-output)))

(ert-deftest sentinel-test-sit-for ()
  (should
   (sentinel-test-wait-function-working-p (lambda () (sit-for 0.01 t)))))

(provide 'sentinel-tests)
